@@ -1,7 +1,7 @@
 '''
 | Author:  Ezio416
 | Created: 2022-08-16
-| Updated: 2022-09-27
+| Updated: 2022-10-01
 
 - Functions for filesystem and path string manipulation
 
@@ -18,13 +18,13 @@ from datetime import datetime as dt
 from fnmatch import fnmatch, fnmatchcase
 from functools import wraps
 import os
-import shutil as sh
-import sys
+from shutil import copy2, copytree, move as shmv, rmtree, unpack_archive
+from time import time
 from zipfile import BadZipFile, ZipFile
 
 from py7zr import exceptions, SevenZipFile, unpack_7zarchive
 
-from .general import gettype, timestamp, unpack
+from .general import secmod_inverse, timestamp, unpack
 
 
 class File():
@@ -279,8 +279,7 @@ def cd(path: str = '..') -> str:
     str
         - path to new current working directory
     '''
-    path = getpath(path)
-    makedirs(path)
+    makedirs(path := getpath(path))
     os.chdir(path)
     return path
 
@@ -299,12 +298,11 @@ def checkwindrive(drive: str) -> str:
     str
         - normalized root path ( 'C:/' ) if valid, else an empty string
     '''
-    drive = forslash(drive).upper()
-    if len(drive) not in (2, 3):
+    if (len_drive := len(drive := forslash(drive).upper())) not in (2, 3):
         return ''
     if drive[0] not in 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' or drive[1] != ':':
         return ''
-    if len(drive) == 3:
+    if len_drive == 3:
         if drive[2] != '/':
             return ''
         return drive
@@ -315,6 +313,7 @@ def checkzip(path: str) -> bool:
     '''
     - checks if an archive file (.7z or .zip) exists and is valid
     - deletes file if invalid or incomplete
+    - does nothing to files with the wrong extension
 
     Parameters
     ----------
@@ -327,47 +326,48 @@ def checkzip(path: str) -> bool:
         - file exists and is valid
 
     False
-        - file doesn't exist, possibly because we deleted it due to corruption
+        - file doesn't exist, because either:
+            - we deleted it
+            - it didn't exist before
     '''
-    path = getpath(path)
     try:
-        if path.endswith('.7z'):
+        if (path := getpath(path)).lower().endswith('.7z'):
             try:
                 with SevenZipFile(path, 'r'):
                     pass
-                return True
+                return True  # 7z file is good
             except exceptions.Bad7zFile:
                 delete(path)
                 return False
-        if path.endswith('.zip'):
+        elif path.lower().endswith('.zip'):
             try:
                 with ZipFile(path):
                     pass
-                return True
+                return True  # zip file is good
             except BadZipFile:
                 delete(path)
                 return False
+        else:
+            raise NotImplementedError(f'unsupported archive format: {path.split(".")[-1]}')
     except FileNotFoundError:
         return False
 
 
 def copymove(func):
     '''
-    - wrapper for :func:`py416.files.copy` and :func:`py416.files.move`, which are very similar, so this handles some of what they share
+    - wrapper for :func:`py416.files.copy` and :func:`py416.files.move`
+    - copy and move are nearly the same, so this handles some of what they share
     '''
     @wraps(func)
     def _copymove(path: str, dest: str, overwrite: bool = False):
-        path = getpath(path)
-        dest = getpath(dest)
-        overwrite = bool(overwrite)
-        if not os.path.exists(path):
+        if not os.path.exists(path := getpath(path)):
             raise FileNotFoundError(f'not found: {path}')
-        if path == dest:
+        if path == (dest := getpath(dest)):
             raise ValueError(f'you can\'t move something into itself: {path}')
         if os.path.exists(dest) and not os.path.isdir(dest):
             raise FileExistsError(f'you can\'t move something into a file: {dest}')
         makedirs(dest)
-        return forslash(func(path, dest, overwrite))
+        return forslash(func(path, dest, bool(overwrite)))
     return _copymove
 
 
@@ -394,17 +394,16 @@ def copy(path: str, dest: str, overwrite: bool = False) -> str:
     str
         - path to copied file/folder
     '''
-    new_path = f'{dest}/{os.path.basename(path)}'
-    new_path_exists = os.path.exists(new_path)
+    new_path_exists = os.path.exists(new_path := f'{dest}/{os.path.basename(path)}')
     if os.path.isfile(path):  # copying file
         if new_path_exists and not overwrite:
             raise FileExistsError(f'destination file already exists: {new_path}')
-        return sh.copy2(path, new_path)
+        return copy2(path, new_path)
     if new_path_exists:  # copying folder
         if overwrite:
-            return sh.copytree(path, new_path, dirs_exist_ok=True)  # overwriting dest folder
+            return copytree(path, new_path, dirs_exist_ok=True)  # overwriting dest folder
         raise FileExistsError(f'destination folder already exists: {new_path}')
-    return sh.copytree(path, new_path)  # dest folder doesn't exist, good
+    return copytree(path, new_path)  # dest folder doesn't exist, good
 
 
 def delete(path: str, force: bool = False) -> None:
@@ -420,13 +419,11 @@ def delete(path: str, force: bool = False) -> None:
         - whether to try `shutil.rmtree() <https://docs.python.org/3/library/shutil.html#shutil.rmtree>`_ to delete a folder and its contents
         - default: False
     '''
-    path = getpath(path)
-    force = bool(force)
-    if not os.path.exists(path):
-        raise FileNotFoundError(f'not found: {path}')
+    if not os.path.exists(path := getpath(path)):
+        return
     if os.path.isdir(path):
-        if force:
-            sh.rmtree(path)
+        if bool(force):
+            rmtree(path)
         else:
             rmdir(path)
     else:
@@ -448,7 +445,7 @@ def forslash(path: str) -> str:
     str
         - path with forward slashes
     '''
-    if gettype(path) != 'str':
+    if type(path) is not str:
         raise TypeError(f'input must be a string; invalid: {path}')
     return path.replace('\\', '/')
 
@@ -481,15 +478,12 @@ def getpath(path: str) -> str:
     str
         - absolute path
     '''
-    parts = list(splitpath(path))
-    if parts == ['']:  # special case
+    if (parts := list(splitpath(path))) == ['']:  # special case
         return ''
     path = joinpath(parts)
-    root = parts[0]
-    if root.startswith('//') or root == '/':  # Windows network location or Unix root
+    if (root := parts[0]).startswith('//') or root == '/':  # Windows network location or Unix root
         return path
-    cwdrive = checkwindrive(root)
-    if cwdrive:  # Windows root
+    if (cwdrive := checkwindrive(root)):  # Windows root
         parts[0] = cwdrive
         return joinpath(parts)
     return f'{getcwd()}/{path}'
@@ -517,23 +511,23 @@ def joinpath(*parts) -> str:
         parts.remove('')  # special case, such as passing 'folder/..' to splitpath()
     if not len(parts):
         return ''  # nothing was passed, or it was cancelled out with '..'
-    if parts[0].startswith('//'):
-        return '/'.join(parts)  # Windows network location
     if parts == ['/']:
         return '/'  # Unix root, alone
     if parts[0] == '/':
         return '/' + '/'.join(parts[1:])  # Unix root, preceding
-    cwdrive = checkwindrive(parts[0])
-    if cwdrive:
+    if parts[0].startswith('//'):
+        return '/'.join(parts)  # Windows network location
+    if (cwdrive := checkwindrive(parts[0])):
         if len(parts) == 1:
             return cwdrive  # Windows drive root, alone
         return cwdrive + '/'.join(parts[1:])  # Windows drive root, preceding
     return '/'.join(parts)
 
 
-def listdir(path: str = '.', dirs: bool = True, files: bool = True, recursive: bool = False, search: str = '', case: bool = False) -> tuple:
+def listdir(path: str = '.', dirs: bool = True, files: bool = True, recursive: bool = False, search: str = '', case: bool = False, recency=0) -> tuple:
     '''
     - lists files/folders within a folder
+    - allows for some filtering by filename and modify date
     - wraps `os.listdir() <https://docs.python.org/3/library/os.html#os.listdir>`_
 
     Parameters
@@ -549,56 +543,74 @@ def listdir(path: str = '.', dirs: bool = True, files: bool = True, recursive: b
     files: bool
         - whether to list files
         - default: True
-    
+
     recursive: bool
         - whether to list all files and folders recursively
         - if False, this will only list files/folders in the specified folder
         - default: False
-    
+
     search: str
-        - kind of like a glob, searches filenames for a specified pattern
+        - like a glob, searches filenames for a specified pattern
         - this does not search any files, rather the list of files we already gathered
         - accepts Unix wildcards ( * ? [...] [!...] )
         - default: nothing
-    
+
     case: bool
         - whether to exactly match capitalization of search term
+        - does nothing if `search` is not set
         - default: False
 
+    recency: float | int | str
+        - how old a file may be, so this only shows the most recently created/modified files
+        - type: float | int
+            - number of seconds
+        - type: str
+            - must be formatted like the base output from :func:`py416.secmod`, i.e. "3d16h5m47s"
+            - can be missing parts, i.e. "3d47s"
+            - capitalization is ignored
+            - if multiple of the same type of value are passed in the string, i.e. "4h16h", only the first value is grabbed
+        - default: 0 (include everything)
+
     Returns
     -------
     tuple[str]
         - absolute paths
     '''
-    path = getpath(path)
-    dirs = bool(dirs)
-    files = bool(files)
-    recursive = bool(recursive)
-    if gettype(search) != 'str':
-        raise TypeError(f'input must be a string; invalid: {search}')
-    if not os.path.exists(path):
+    if not os.path.exists(path := getpath(path)):
         return ()
+    if type(search) is not str:
+        raise TypeError(f'input must be a string; invalid: {search}')
+    if (recency_type := type(recency)) not in (float, int, str):
+        raise TypeError(f'input must be a number or string; invalid: {recency}')
     result = []
     for child in os.listdir(path):
         child = joinpath(path, child)
         if os.path.isdir(child):
-            if dirs:
+            if bool(dirs):
                 result.append(child)
-            if recursive:
+            if bool(recursive):
                 result += list(listdir(child, dirs=dirs, files=files, recursive=True))
-        elif files:
+        elif bool(files):
             result.append(child)
     if search:
-        result2 = []
+        tmp = []
         for fpath in result:
             name = os.path.basename(fpath)
-            if case:
+            if bool(case):
                 if fnmatchcase(name, search):
-                    result2.append(fpath)
+                    tmp.append(fpath)
                 continue
             if fnmatch(name, search):
-                result2.append(fpath)
-        return tuple(result2)
+                tmp.append(fpath)
+        result = tmp
+    if recency:
+        tmp = []
+        recency = secmod_inverse(recency) if recency_type is str else float(recency)
+        now = time()
+        for fpath in result:
+            if now - os.path.getmtime(fpath) < recency:
+                tmp.append(fpath)
+        return tuple(tmp)
     return tuple(result)
 
 
@@ -623,23 +635,13 @@ def log(path: str, msg: str, ts: bool = True, ts_args: tuple = (1, 0, 1, 1, 1, 0
         - arguments to pass to :func:`py416.timestamp`
         - default return example: [2022-08-19 13:24:54 -06:00]
     '''
-    path = getpath(path)
-    if gettype(msg) != 'str':
+    if type(msg) is not str:
         raise ValueError(f'input must be a string; invalid: {msg}')
-    ts = bool(ts)
-    if gettype(ts_args) not in ('list', 'tuple'):
+    if type(ts_args) not in (list, tuple):
         raise ValueError(f'input must be a list/tuple; invalid: {ts_args}')
-    makedirs(parent(path))
-    now = timestamp(*ts_args) + '  ' if ts else ''
-    orig_stdout = sys.stdout
-    try:
-        with open(path, 'a') as file:
-            sys.stdout = file
-            print(f'{now}{msg}')
-    except Exception:
-        pass
-    finally:
-        sys.stdout = orig_stdout
+    makedirs(parent(path := getpath(path)))
+    with open(path, 'a') as file:
+        file.write(f'{timestamp(*ts_args) + "  " if bool(ts) else ""}{msg}\n')
 
 
 def makedirs(*dirs, ignore_errors: bool = True) -> tuple:
@@ -649,9 +651,11 @@ def makedirs(*dirs, ignore_errors: bool = True) -> tuple:
 
     Parameters
     ----------
-    dirs
-        - str: path to folder
-        - iterable: nestings of lists and tuples with folder paths
+    dirs: list | str | tuple
+        - type: str
+            - path to folder
+        - type: list | tuple
+            - nestings of lists and tuples with folder path strings as base elements
 
     ignore_errors: bool
         - whether to catch all Exceptions from :func:`os.makedirs()`
@@ -663,20 +667,18 @@ def makedirs(*dirs, ignore_errors: bool = True) -> tuple:
     tuple
         - folders we failed to create
     '''
-    if gettype(dirs) not in ('list', 'str', 'tuple'):
+    if type(dirs) not in (list, str, tuple):
         raise TypeError(f'input must be a string/list/tuple; invalid: {dirs}')
-    ignore_errors = bool(ignore_errors)
     errored = []
     for dir in unpack(dirs):
-        if gettype(dir) != 'str':
+        if type(dir) is not str:
             errored.append(dir)
             continue
-        dir = getpath(dir)
-        if not os.path.exists(dir):
+        if not os.path.exists(dir := getpath(dir)):
             try:
                 os.makedirs(dir)
             except Exception:
-                if not ignore_errors:
+                if not bool(ignore_errors):
                     raise
                 errored.append(dir)
     return tuple(errored)
@@ -705,11 +707,9 @@ def makefile(path: str, msg: str = '', overwrite: bool = False) -> str:
     str
         - path to new file
     '''
-    path = getpath(path)
-    if gettype(msg) != 'str':
-        raise TypeError(f'input must be a string; invalid: {path}')
-    overwrite = bool(overwrite)
-    if os.path.exists(path) and overwrite:
+    if type(msg) is not str:
+        raise TypeError(f'input must be a string; invalid: {msg}')
+    if os.path.exists(path := getpath(path)) and bool(overwrite):
         delete(path, force=True)
     if os.path.isfile(path):
         raise FileExistsError(f'destination file already exists: {path}')
@@ -744,21 +744,20 @@ def move(path: str, dest: str, overwrite: bool = False) -> str:
     str
         - path to moved file/folder
     '''
-    new_path = f'{dest}/{os.path.basename(path)}'
-    new_path_exists = os.path.exists(new_path)
+    new_path_exists = os.path.exists(new_path := f'{dest}/{os.path.basename(path)}')
     if os.path.isfile(path):  # moving file
         if new_path_exists:
             if not overwrite:
                 raise FileExistsError(f'destination file already exists: {new_path}')
             delete(new_path)  # deleting dest file to overwrite it
-        return sh.move(path, dest)
+        return shmv(path, dest)
     if new_path_exists:  # moving folder
         if overwrite:
-            result = sh.copytree(path, new_path, dirs_exist_ok=True)  # overwriting dest folder
+            result = copytree(path, new_path, dirs_exist_ok=True)  # overwriting dest folder
             delete(path, force=True)  # deleting original (we're doing copy->delete manually)
             return result
         raise FileExistsError(f'destination folder already exists: {new_path}')
-    return sh.move(path, dest)  # dest folder doesn't exist, good
+    return shmv(path, dest)  # dest folder doesn't exist, good
 
 
 def parent(path: str) -> str:
@@ -775,8 +774,7 @@ def parent(path: str) -> str:
     str
         - parent path
     '''
-    path = getpath(path)
-    if not path:
+    if not (path := getpath(path)):
         return ''
     if any([path == '/',  # Root
             checkwindrive(path),
@@ -803,13 +801,11 @@ def rename(path: str, name: str) -> str:
     str
         - new path to file/folder
     '''
-    path = getpath(path)
-    if gettype(name) != 'str':
-        raise TypeError(f'input must be a string; invalid: {name}')
-    if not os.path.exists(path):
+    if not os.path.exists(path := getpath(path)):
         raise FileNotFoundError(f'not found: {path}')
-    newpath = f'{parent(path)}/{name}'
-    os.rename(path, newpath)
+    if type(name) is not str:
+        raise TypeError(f'input must be a string; invalid: {name}')
+    os.rename(path, (newpath := joinpath(parent(path), name)))
     return newpath
 
 
@@ -832,19 +828,17 @@ def rmdir(path: str, delroot: bool = True) -> int:
     int
         - number of deleted folders
     '''
-    path = getpath(path)
-    delroot = bool(delroot)
-    if not os.path.exists(path):
+    if not os.path.exists(path := getpath(path)):
         raise FileNotFoundError(f'not found: {path}')
     if not os.path.isdir(path):
-        raise ValueError(f'not a directory: {path}')
+        raise ValueError(f'not a folder: {path}')
     count = 0
     for item in listdir(path):
         if os.path.isdir(item):
             count += rmdir(item)
-    if not len(listdir(path)) and delroot:
+    if not len(listdir(path)) and bool(delroot):
         if path == getcwd():
-            cd()  # we're in the directory we're trying to delete, so go up
+            cd()  # we're in the folder we're trying to delete, so go up
         os.rmdir(path)
         count += 1
     return count
@@ -867,17 +861,14 @@ def splitpath(path: str) -> tuple:
     '''
     if not path:
         return '',
-    path = forslash(path)
-    win_net = False
-
-    if path in ('/', '/.', '/..'):  # special cases
+    if (path := forslash(path)) in ('/', '/.', '/..'):  # special cases
         return '/',
-    parts = path.split('/')
+    win_net = False
     if path == '.':  # current directory
         path = getcwd()
     elif path == '..':  # parent of current directory
         path = forslash(os.path.dirname(getcwd()))
-    elif parts[-1] == '.':  # path/. is just path
+    elif (parts := path.split('/'))[-1] == '.':  # folder/. is just folder
         path = path[:-2]
     elif parts[-1] == '..':  # parent of path
         if len(parts) == 2:  # 'folder/..'
@@ -889,15 +880,13 @@ def splitpath(path: str) -> tuple:
         if win_net:
             path = '//' + path
     parts = path.split('/')
-
     if not parts[0]:  # Unix root
         parts[0] = '/'
-    cwdrive = checkwindrive(parts[0])
-    if cwdrive:  # Windows drive root
+    if (cwdrive := checkwindrive(parts[0])):  # Windows drive root
         parts[0] = cwdrive
     if path.startswith('//'):  # Windows network location
         result = [f'//{parts[2]}']
-        return tuple(result + [a for a in parts[3:]]) if len(parts) > 2 else tuple(result)
+        return tuple(result + parts[3:]) if len(parts) > 2 else tuple(result)
     if len(parts) == 2:
         if parts[1] == '':
             return parts[0],
@@ -917,17 +906,16 @@ def unzip(path: str, remove: bool = False) -> None:
         - whether to delete archive after unzipping
         - default: False
     '''
-    path = getpath(path)
-    remove = bool(remove)
-    if not os.path.exists(path):
+    if not os.path.exists(path := getpath(path)):
         raise FileNotFoundError(f'not found: {path}')
+    remove = bool(remove)
     fparent = parent(path)
     if path.endswith('.7z'):
         unpack_7zarchive(path, fparent)
         if remove:
             delete(path)
     elif path.endswith(('.gz', '.rar', '.tar', '.zip')):
-        sh.unpack_archive(path, fparent)
+        unpack_archive(path, fparent)
         if remove:
             delete(path)
     else:
@@ -955,22 +943,18 @@ def unzipdir(path: str, ignore_errors: bool = True) -> int:
     int
         - number of unzipped archives
     '''
-    path = getpath(path)
-    ignore_errors = bool(ignore_errors)
-    if not os.path.exists(path):
+    if not os.path.exists(path := getpath(path)):
         raise FileNotFoundError(f'not found: {path}')
     unzipped = 0
     while True:
         unzipped_this_run = 0
         for file in listdir(path, dirs=False):
             try:
-                if not unzip(file, remove=True):
-                    unzipped += 1
-                    unzipped_this_run += 1
-            except ModuleNotFoundError:
-                raise
+                unzip(file, remove=True)
+                unzipped += 1
+                unzipped_this_run += 1
             except Exception:
-                if not ignore_errors:
+                if not bool(ignore_errors):
                     raise
         if not unzipped_this_run:
             break
